function [ fh ] = plotFeasibleSetOfGSIP1D(problem,indicesFreeParameters,valuesFixedParameters,lowerBounds,upperBounds,nrPointsInDir)

%initialize figure
fh = fig('width',14, 'height', 5);
ah = axes('Parent',fh);
set(ah,'XGrid','on');
hold on


%necessary data
NrSemiInfConstrs=problem.getNrSemiInfConstrs();
NrPointsInDir=nrPointsInDir;
%Settings for the solver
myLLPfminconOptions = optimset('Algorithm', 'sqp');
myLLPfminconOptions = optimset(myLLPfminconOptions, 'Display', 'off');
myLLPfminconOptions = optimset(myLLPfminconOptions, 'MaxFunEvals', Inf);
myLLPfminconOptions = optimset(myLLPfminconOptions, 'MaxIter', 1000);
myLLPfminconOptions = optimset(myLLPfminconOptions, 'FunValCheck', 'on');
%myLLPfminconOptions = optimset(myLLPfminconOptions, 'GradObj', 'on', 'GradConstr', 'on', 'DerivativeCheck', der_check);
myLLPfminconOptions = optimset(myLLPfminconOptions, 'ScaleProblem', 'obj-and-constr');
myLLPfminconOptions = optimset(myLLPfminconOptions, 'FinDiffType', 'central');




%Compute x-Point
x1=linspace(lowerBounds,upperBounds,NrPointsInDir);
x_all=[repmat(valuesFixedParameters(1:indicesFreeParameters-1),NrPointsInDir,1),...
    x1',repmat(valuesFixedParameters(indicesFreeParameters:end),NrPointsInDir,1)];

%optimization of every lower level problem
phi=-inf*ones(NrPointsInDir,1);
for j=1:NrPointsInDir;
    x=x_all(j,:);
    for i=1:NrSemiInfConstrs
        k=problem.getIndexSetOfSemiInfConstr(i);   
        Yint = problem.getInitValsLowLevVars(k,x);
        [lbl, ubl] = problem.getBoundsOnLowLevVars(k,x);
        [Aineq,bineq] = problem.getLinIneqsOnLowLevVars (k,x);
        % max-problem 
        [~,fval] = fmincon(@LLPObjFun,Yint,Aineq,bineq,[],[],lbl,ubl,@LLPConFuns,myLLPfminconOptions,problem,i,x,k);
        phi(j)=max(phi(j),-fval);

        %compute non semi-infinite constraints
        %collect bounds
        [problb,probub]=problem.getBoundsOnUppLevVars();
        
        phi(j)=max([phi(j),problb-x]); 
        phi(j)=max([phi(j),x-probub]);
        
        %collect linear constraints
        [A,b]=problem.getLinIneqsOnUppLevVars();
        for i2=1:size(A,1)
            phi(j)=max(phi(j),x*A(i2,:)'-b(i2));
        end
        
        [A,b]=problem.getLinEqsOnUppLevVars();
        if size(A,1)>0||problem.getNrUppLevNonlinEqs>0
            warning('This plot does not work for equality constraints. We ignore the equality constraints');            
        end
        if problem.getNrUppLevNonlinIneqs>0
            phi(j)=max([phi(j);problem.evalUppLevNonlinIneqConstrs(x)]);
        end
        
        %display every 10th evalutation of the function phi
       
    end
    if mod(j,10)==0
        fprintf('The first %i points are computed\n',j);
    end 
end



phiPosNeg=phi>0;
Index=1;
crit=true; 
while crit
    %determine the severel Intervals
    
    %leftBound first non negative value
    [minleft,leftIndex]=min(phiPosNeg);
    phiPosNeg=phiPosNeg(leftIndex+1:end);
    leftIndex=Index+leftIndex;
    Index=leftIndex;
    if minleft==1
        break
    end

    %rigtht Bound last non negative value
    [maxright,rightIndex]=max(phiPosNeg);
    phiPosNeg=phiPosNeg(rightIndex:end);
    rightIndex=rightIndex+Index-1;
    Index=rightIndex;

    %plot the detected Interval
    if maxright==0
        rightIndex=NrPoints;
    else
        %plot right bracket !!Attention special cases
        bracketIndex=max(leftIndex,rightIndex-round(1/20*NrPointsInDir));
        plot([x1(bracketIndex),x1(rightIndex),x1(rightIndex),x1(bracketIndex)],[0.25,0.25,0.75,0.75],'-b');
    end
    h=fill([x1(leftIndex),x1(leftIndex),x1(rightIndex),x1(rightIndex)],[0.25,0.75,0.75,0.25],[135/255, 206/255, 250/255]);
    set(h, 'EdgeColor', 'none','facealpha',0.5);
    %plot left bracket !!Attention special cases
    if leftIndex~=1
        bracketIndex=min(leftIndex+round(1/20*NrPointsInDir),rightIndex);
        plot([x1(bracketIndex),x1(leftIndex),x1(leftIndex),x1(bracketIndex)],[0.25,0.25,0.75,0.75],'-b');
    end
    %check criterion
    crit=rightIndex~=NrPointsInDir;
end


%%plot the line for the Real line
plot([lowerBounds,upperBounds],[0.5,0.5],'-','color','black');
set(gca,'YTickLabel',[]);
ylabel('');
xlabel(sprintf('x%i',indicesFreeParameters));
xlim([lowerBounds(1), upperBounds(1)]);
ylim([0, 1]);
title('Feasible set of GSIP');
hold off


%Functions for the optimization

function [g, grad_g_y] = LLPObjFun(y,GSIP_problem2,i,xval,~)

    g_temp = GSIP_problem2.evalSemiInfConstrs(i,xval,y);
    g=-g_temp; %max-problem


    grad_g_y_temp = GSIP_problem2.DySemiInfConstrs(i,xval,y);
    grad_g_y=-grad_g_y_temp';
end

%constraints of the lower level problem
function [u, h, grad_u, grad_h] = LLPConFuns(y,GSIP_problem2,~,xval,k2)

    h = [];
    grad_h = [];
    assert(size(y,1)==1);
    u = GSIP_problem2.evalLowLevNonlinConstrs(k2,xval,y);

    grad_u = GSIP_problem2.DyLowLevNonlinConstrs(k2,xval,y)';


end






end

