function [ h ] = plotFeasibleSetOfGSIP3D(problem,indicesFreeParameters,valuesFixedParameters,lowerBounds,upperBounds,nrPointsInDir)
%PLOTFEASIBLESETOFGSIP3D Summary of this function goes here
%   Detailed explanation goes here

%necessary data
NrSemiInfConstrs=problem.getNrSemiInfConstrs();
NrPointsInDir=nrPointsInDir;
if NrPointsInDir>10
    warning('''NrPointsInDir^3'' lower level problems have to be solved. This may take some time')
end

h=figure;
hold on
%Settings for the solver
myLLPfminconOptions = optimset('Algorithm', 'sqp');
myLLPfminconOptions = optimset(myLLPfminconOptions, 'Display', 'off');
myLLPfminconOptions = optimset(myLLPfminconOptions, 'MaxFunEvals', Inf);
myLLPfminconOptions = optimset(myLLPfminconOptions, 'MaxIter', 1000);
myLLPfminconOptions = optimset(myLLPfminconOptions, 'FunValCheck', 'on');
%myLLPfminconOptions = optimset(myLLPfminconOptions, 'GradObj', 'on', 'GradConstr', 'on', 'DerivativeCheck', der_check);
myLLPfminconOptions = optimset(myLLPfminconOptions, 'ScaleProblem', 'obj-and-constr');
myLLPfminconOptions = optimset(myLLPfminconOptions, 'FinDiffType', 'central');

assert(problem.getNrUppLevVars==3);
%Plot the optimal value function
%ezimplot3(@optimialValueFunction,[lowerBounds(1),UpperBounds(1),lowerBounds(2),UpperBounds(2),lowerBounds(3),UpperBounds(3)]);

%check wether the indices are in the right order
[indicesFreeParameters,I]=sort(indicesFreeParameters);
lowerBounds=lowerBounds(I);
upperBounds=upperBounds(I);

for j=1:NrPointsInDir
    for j2=1:NrPointsInDir
        for j3=1:NrPointsInDir
            %Compute x-Point
            x1=lowerBounds(1)+(NrPointsInDir-1-j3+1)/(NrPointsInDir-1)*(upperBounds(1)-lowerBounds(1));
            x2=lowerBounds(2)+(NrPointsInDir-1-j2+1)/(NrPointsInDir-1)*(upperBounds(2)-lowerBounds(2));
            x3=lowerBounds(3)+(NrPointsInDir-1-j+1)/(NrPointsInDir-1)*(upperBounds(3)-lowerBounds(3));
            x=[valuesFixedParameters(1:indicesFreeParameters(1)-1),x1,...
                valuesFixedParameters(indicesFreeParameters(1):indicesFreeParameters(2)-2),x2,...
                valuesFixedParameters(indicesFreeParameters(2)-1:indicesFreeParameters(3)-3),x3,...
                valuesFixedParameters(indicesFreeParameters(3)-2:end)];
            %optimization of every lower level problem
            phi=-inf;
            for i=1:NrSemiInfConstrs
                k=problem.getIndexSetOfSemiInfConstr(i);   
                Yint = problem.getInitValsLowLevVars(k,x);
                [lbl, ubl] = problem.getBoundsOnLowLevVars(k,x);

                [Aineq,bineq] = problem.getLinIneqsOnLowLevVars (k,x);
                % max-problem    
                [~,fval] = fmincon(@LLPObjFun,Yint,Aineq,bineq,[],[],lbl,ubl,@LLPConFuns,myLLPfminconOptions,problem,i,x,k);
                phi=max(phi,-fval);               
            end
            % non semi infinite constraints
            
            %compute non semi-infinite constraints
            %collect bounds
            [problb,probub]=problem.getBoundsOnUppLevVars();
            phi=max(phi,problb(1)-x1); 
            phi=max(phi,x1-probub(1));
            phi=max(phi,problb(2)-x2); 
            phi=max(phi,x2-probub(2));
            phi=max(phi,problb(3)-x3); 
            phi=max(phi,x3-probub(3));
            
            
            %collect linear constraints
            [A,b]=problem.getLinIneqsOnUppLevVars();
            for i2=1:size(A,1)
                phi=max(phi,x*A(i2,:)'-b(i2));
            end

            [A,b]=problem.getLinEqsOnUppLevVars();
            if size(A,1)>0||problem.getNrUppLevNonlinEqs>0
                warning('This plot does not work for equality constraints. We ignore the equality constraints');            
            end
            if problem.getNrUppLevNonlinIneqs>0
                phi=max([phi;problem.evalUppLevNonlinIneqConstrs(x)]);
            end
        
            
            
            
            
            if phi<=0.01
                plot3(x1,x2,x3,'.','color',[min(max(0,-phi),0.8),min(max(0,-phi),0.8),1],'LineWidth',50);
                %plot3(x1,x2,x3,'.','color',[(x3-lowerbounds(3))/(upperbounds(3)-lowerbounds(3))*0.8,(x3-lowerbounds(3))/(upperbounds(3)-lowerbounds(3)),1],'LineWidth',50);
            end
            %display every 10th evalutation of the function phi
            if mod((j-1)*NrPointsInDir^2+(j2-1)*NrPointsInDir+j3,10)==0
                fprintf('The first %i points are computed\n',(j-1)*NrPointsInDir^2+(j2-1)*NrPointsInDir+j3);
            end
        end
    end
end

%settings on figure
xlabel(sprintf('x%i',indicesFreeParameters(1)));
ylabel(sprintf('x%i',indicesFreeParameters(2)));
zlabel(sprintf('x%i',indicesFreeParameters(3)));
xlim([lowerBounds(1),upperBounds(1)]);
ylim([lowerBounds(2),upperBounds(2)]);
zlim([lowerBounds(3),upperBounds(3)]);
view([lowerBounds(1),lowerBounds(2),upperBounds(3)]);
title('Feasible set of GSIP')

    function [g, grad_g_y] = LLPObjFun(y,GSIP_problem2,i,xval,~)

        g_temp = GSIP_problem2.evalSemiInfConstrs(i,xval,y);
        g=-g_temp; %max-problem


        grad_g_y_temp = GSIP_problem2.DySemiInfConstrs(i,xval,y);
        grad_g_y=-grad_g_y_temp';
    end

    %constraints of the lower level problem
    function [u, h, grad_u, grad_h] = LLPConFuns(y,GSIP_problem2,~,xval,k2)

        h = [];
        grad_h = [];
        assert(size(y,1)==1);
        u = GSIP_problem2.evalLowLevNonlinConstrs(k2,xval,y);

        grad_u = GSIP_problem2.DyLowLevNonlinConstrs(k2,xval,y)';


    end


end

