classdef Problem2<ImplicitGSIPProblem
    % p=Problem2() creates an instance of the GSIP problem:
    %
    %       min   x1+x2
    %
    %       s.t.  y-x1-x2<=0 for all y in Y(x) 
    %             -2x1-x2<=-1-10^-6
    %       where:
    %
    %       Y(x)={y in R with
    %               y   <= x1-2
    %               -2y <= x2+3 }
    
    
    
    %We have chosen sum(x) as the objective fucntion. The feasible set is
    %is restricted by one more constraint.   
    properties(GetAccess=protected)

        
               
        nrOfUppLevVars=2;
        nrOfSemiInfConstrs=1; 
        
        nrOfNonlinEqsUppLev=0;                
        nrOfNonlinIneqsUppLev=0;
        
        linEqsOnUppLevVars=[];                     
        linIneqsOnUppLevVars=[-2,-1, -1-1.e-6];  
            
        boundsOnUppLevVars=[-inf,-inf;inf,inf];     
                              
        initValsOfUppLevVars=[0,0];                
        
        nrOfLowLevVars=1;                          
        nrOfLowLevNonlinConstrs=0;                        
        
        con2LowLev=1;
        nrOfIndexset=1;
        
    end
    
   
    methods 
        %objective function
        function f=evalObjFun(~,x)                   
            f=sum(x);
        end
        function grad_f=gradObjFun(~,~)                    
            grad_f=[1;1];
        end
        
        %upper level
        function g=evalSemiInfConstrs(~,~,x,y)        
                                                         
            g=y-x(1) -x(2);            
        end
        function Dx_g=DxSemiInfConstrs(~,~,~,y)      
            m=size(y,1);
            Dx_g=-ones(m,2);                
        end
        function Dy_g=DySemiInfConstrs(~,~,~,y)
            m=size(y,1);
            Dy_g=ones(m,1);                
        end
               
            
        function y=getInitValsLowLevVars(~,~,x)           
            y=1/2*(-3/2-x(2)/2+x(1)-2);                
        end
        function Dx_y=DxStartingPointLowLev(~,~,~)
            Dx_y=[-1/4, 1/2];
        end
                
        %Linear inequalities
        function [A,b]=getLinIneqsOnLowLevVars(obj,i,x)
            A=[1;-2];
            b=[-2+x(1);3+x(2)];
        end
        
        %transformation
        function y=evalTrafoFun(obj,i,x,z)
            y=z*(-3/2-x(2)*0.5) +(1-z)*(x(1)-2);
        end
        function z=evalInvTrafoFun(obj,i,x,y)
            z=(y-x(1)+2)/(-3/2-x(2)/2-x(1)+2);
        end
        function Dx_y=DxTrafoFun(~,~,x,z)
            Dx_y=zeros(1,size(x,2),size(z,1));
            o=ones(size(z));
          
            Dx_y(1,:,:)=[(1-z),-z*o*0.5]';   
         
        end
    end
    
    
end

