classdef Problem23 < ImplicitGSIPProblem
    % p=Problem23 creates an instance of the GSIP problem:
    %
    %       min   x2^2-4x2
    %
    %       s.t.  x1cos(y)+x2sin(y)-1 <=0
    %             0 <= x1 <= 2
    %             0 <= x2 <= 2
    %
    %       where:
    %
    %       Y(x)={y in R with
    %               -y^2-1.75x2+5.75<=0 
    %               0 <= y1 <= pi}
    
    properties(GetAccess=protected)
        
        nrOfUppLevVars=2;                         
        boundsOnUppLevVars=[0,0;2,2];              
        nrOfSemiInfConstrs=1;                      
        initValsOfUppLevVars=[0,0];               
        
        linEqsOnUppLevVars=[];                    
        linIneqsOnUppLevVars=[];                  
        nrOfNonlinEqsUppLev=0;               
        nrOfNonlinIneqsUppLev=0;             
        
        
        nrOfLowLevVars=1;                         
        nrOfLowLevNonlinConstrs=1;                       
        
        con2LowLev=1;
        nrOfIndexset=1;
        
    end
    
    
    methods
        %objective function
        function f=evalObjFun(obj,x)                    
            f=x(2)^2-4*x(2);
        end
        function grad_f=gradObjFun(obj,x)                    
            grad_f=[0;2*x(2)-4];
        end
        
        %upper level
        function g=evalSemiInfConstrs(obj,i,x,y)    
                                                       
            g=x(1)*cos(y)+x(2)*sin(y)-1;            
        end
        function Dx_g=DxSemiInfConstrs(obj,i,x,y)   

            Dx_g=[cos(y),sin(y)];                
        end
        function Dy_g=DySemiInfConstrs(obj,i,x,y)
            Dy_g=[-x(1)*sin(y)+x(2)*cos(y)];                
        end
               
        
               
        %lower level
        function u=evalLowLevNonlinConstrs(obj,~,x,y)           
            u=[-y^2-7/4*x(2)+23/4];                        
        end
        function Dy_u=DyLowLevNonlinConstrs(obj,~,x,y)             
            Dy_u=[-2*y];                        
        end
        function Dx_u=DxLowLevNonlinConstrs(obj,~,x,y)             
            Dx_u=[0,-7/4];
        end
            
        function y=getInitValsLowLevVars(obj,~,x)          
            [lb,ub] = obj.getInterval(x);
            y=(lb+ub)/2;                
        end
        function Dx_y=DxStartingPointLowLev(obj,~,x)
            [Dx_lb,Dx_ub]=DxInterval(x);
            Dx_y=(Dx_lb+Dx_ub)/2;
        end
        
        function [lb,ub]=getBoundsOnLowLevVars(obj,~,x)   
             lb=0;
             ub=pi;
        end
                                
        
        %transformation
        function y=evalTrafoFun(obj,i,x,z)
            [lb,ub]=obj.getInterval(x);
            y=lb*z+ub*(1-z);
        end
        function z=evalInvTrafoFun(obj,i,x,y)
            [lb,ub]=obj.getInterval(x);
            z=(y-ub)/((lb-ub));
                        
        end
        function Dx_y=DxTrafoFun(obj,i,x,z)
            [Dx_lb,Dx_ub]=obj.DxInterval(x);
            Dx_y=zeros(1,size(x,2),size(z,1));
            o=ones(size(z));
          
            Dx_y(1,:,:)=(o*Dx_lb*z+(1-z)*(Dx_ub))';
        end
        
    end
    methods(Access=private)
        function [lb,ub] = getInterval(obj,x)           
            lb=sqrt(23/4-7/4*x(2));
            ub=pi;
        end
        function [Dx_lb,Dx_ub]=DxInterval(obj,x)
            
            Dx_lb=[0,-7/8*1/sqrt(23/4-7/4*x(2))];  
           
            
            Dx_ub=[0,0];
        end
    end
    
end
